<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron" xmlns:u="utils"
        schemaVersion="iso" queryBinding="xslt2">

  <title>Rules for PLPEF BIS 3.0 </title>
  <ns prefix="ext" uri="urn:oasis:names:specification:ubl:schema:xsd:CommonExtensionComponents-2"/>
  <ns uri="urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2" prefix="cbc"/>
  <ns uri="urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2" prefix="cac"/>
  <ns prefix="cbc-pl" uri="urn:pl:extended:CommonBasicComponents-2"/>
  <ns prefix="cac-pl" uri="urn:pl:extended:CommonAggregateComponents-2"/>
  <ns uri="urn:oasis:names:specification:ubl:schema:xsd:CreditNote-2" prefix="ubl-creditnote"/>
  <ns uri="urn:oasis:names:specification:ubl:schema:xsd:Invoice-2" prefix="ubl-invoice"/>
  <ns uri="http://www.w3.org/2001/XMLSchema" prefix="xs"/>
  <ns uri="utils" prefix="u"/>

  <!-- Parameters -->

  <let name="profile" value="if (/*/cbc:ProfileID and matches(normalize-space(/*/cbc:ProfileID), 'urn:fdc:www.efaktura.gov.pl:ver2.0:corr_inv:ver4.0')) then tokenize(normalize-space(/*/cbc:ProfileID), ':')[7] else 'Unknown'"/>
  <let name="supplierCountry" value="if (/*/cac:AccountingSupplierParty/cac:Party/cac:PostalAddress/cac:Country/cbc:IdentificationCode) then upper-case(normalize-space(/*/cac:AccountingSupplierParty/cac:Party/cac:PostalAddress/cac:Country/cbc:IdentificationCode)) else 'XX'"/>

  <!-- -->

  <let name="documentCurrencyCode" value="/*/cbc:DocumentCurrencyCode"/>

  <!-- Functions -->

  <function xmlns="http://www.w3.org/1999/XSL/Transform" name="u:slack" as="xs:boolean">
    <param name="exp" as="xs:decimal"/>
    <param name="val" as="xs:decimal"/>
    <param name="slack" as="xs:decimal"/>
    <sequence select="xs:decimal($exp + $slack) &gt;= $val and xs:decimal($exp - $slack) &lt;= $val"/>
  </function>

  <!-- Empty elements -->
  <pattern>
    <rule context="cbc:*">
      <assert id="PLPEF-EN16931-R008"
              test=". != ''"
              flag="fatal">Document MUST not contain empty elements.</assert>
    </rule>
    <rule context="cac:*">
      <assert id="PLPEF-EN16931-R009"
              test="count(*) != 0"
              flag="fatal">Document MUST not contain empty elements.</assert>
    </rule>
  </pattern>

  <!--
    Transaction rules

    R00X - Document level
    R01X - Accounting customer
    R02X - Accounting supplier
    R04X - Allowance/Charge (document and line)
    R05X - Tax
    R06X - Payment
    R1XX - Line level
    R11X - Invoice period
  -->
  <pattern>

    <!-- Document level -->
    <rule context="ubl-creditnote:CreditNote | ubl-invoice:Invoice">
      <assert id="PLPEF-EN16931-R001"
              test="cbc:ProfileID"
              flag="fatal">Business process MUST be provided.</assert>
      <assert id="PLPEF-EN16931-R002"
              test="count(cbc:Note) &lt;= 1"
              flag="fatal">No more than one note is allowed on document level.</assert>
      <assert id="PLPEF-EN16931-R003"
              test="cbc:BuyerReference or cac:OrderReference/cbc:ID"
              flag="fatal">A buyer reference or purchase order reference MUST be provided.</assert>
      <assert id="PLPEF-EN16931-R004"
              test="starts-with(normalize-space(cbc:CustomizationID/text()), 'urn:fdc:www.efaktura.gov.pl:ver2.0:trns:corr_inv:ver4.0')"
              flag="fatal">Specification identifier MUST have the value 'urn:fdc:www.efaktura.gov.pl:ver2.0:trns:corr_inv:ver4.0'.</assert>

      <assert id="PLPEF-EN16931-R053"
              test="count(cac:TaxTotal[cac:TaxSubtotal]) = 1"
              flag="fatal">Only one tax total with tax subtotals MUST be provided.</assert>
      <assert id="PLPEF-EN16931-R054"
              test="count(cac:TaxTotal[not(cac:TaxSubtotal)]) = (if (cbc:TaxCurrencyCode) then 1 else 0)"
              flag="fatal">Only one tax total without tax subtotals MUST be provided when tax currency code is provided.</assert>
    </rule>

    <rule context="cbc:TaxCurrencyCode">
      <assert id="PLPEF-EN16931-R005"
              test="not(normalize-space(text()) = normalize-space(../cbc:DocumentCurrencyCode/text()))"
              flag="fatal">VAT accounting currency code MUST be different from invoice currency code when provided.</assert>
    </rule>

    <!-- Accounting customer -->
    <rule context="cac:AccountingCustomerParty/cac:Party">
      <assert id="PLPEF-EN16931-R010"
              test="cbc:EndpointID"
              flag="fatal">Buyer electronic address MUST be provided</assert>
    </rule>

    <!-- Accounting supplier -->
    <rule context="cac:AccountingSupplierParty/cac:Party">
      <assert id="PLPEF-EN16931-R020"
              test="cbc:EndpointID"
              flag="fatal">Seller electronic address MUST be provided</assert>
    </rule>

    <!-- Allowance/Charge (document level/line level) -->
    <rule context="ubl-invoice:Invoice/cac:AllowanceCharge[cbc:MultiplierFactorNumeric and not(cbc:BaseAmount)] | ubl-invoice:Invoice/cac:InvoiceLine/cac:AllowanceCharge[cbc:MultiplierFactorNumeric and not(cbc:BaseAmount)] | ubl-creditnote:CreditNote/cac:AllowanceCharge[cbc:MultiplierFactorNumeric and not(cbc:BaseAmount)] | ubl-creditnote:CreditNote/cac:CreditNoteLine/cac:AllowanceCharge[cbc:MultiplierFactorNumeric and not(cbc:BaseAmount)]">
      <assert id="PLPEF-EN16931-R041"
              test="false()"
              flag="fatal">Allowance/charge base amount MUST be provided when allowance/charge percentage is provided.</assert>
    </rule>
    <rule context="ubl-invoice:Invoice/cac:AllowanceCharge[not(cbc:MultiplierFactorNumeric) and cbc:BaseAmount] | ubl-invoice:Invoice/cac:InvoiceLine/cac:AllowanceCharge[not(cbc:MultiplierFactorNumeric) and cbc:BaseAmount] | ubl-creditnote:CreditNote/cac:AllowanceCharge[not(cbc:MultiplierFactorNumeric) and cbc:BaseAmount] | ubl-creditnote:CreditNote/cac:CreditNoteLine/cac:AllowanceCharge[not(cbc:MultiplierFactorNumeric) and cbc:BaseAmount]">
      <assert id="PLPEF-EN16931-R042"
              test="false()"
              flag="fatal">Allowance/charge percentage MUST be provided when allowance/charge base amount is provided.</assert>
    </rule>
    <rule context="ubl-invoice:Invoice/cac:AllowanceCharge | ubl-invoice:Invoice/cac:InvoiceLine/cac:AllowanceCharge | ubl-creditnote:CreditNote/cac:AllowanceCharge | ubl-creditnote:CreditNote/cac:CreditNoteLine/cac:AllowanceCharge">
 
      <let name="rChargeIndicator" value="cbc:ChargeIndicator"/>
      <let name="rAllowanceChargeReasonCode" value="cbc:AllowanceChargeReasonCode"/>
      <let name="rAllowanceChargeReason" value="cbc:AllowanceChargeReason"/>
      <let name="rID" value="cac:TaxCategory/cbc:ID"/>
      <let name="rPercent" value="cac:TaxCategory/cbc:Percent"/>
      <let name="riID" value="../cac:Item/cac:ClassifiedTaxCategory/cbc:ID"/>
      <let name="riPercent" value="../cac:Item/cac:ClassifiedTaxCategory/cbc:Percent"/>
      
    <assert id="PEPPOL-EN16931-R040" test="
		  if(//ext:UBLExtensions/ext:UBLExtension/ext:ExtensionContent/cac-pl:AdditionalInvoiceGrossData/cbc-pl:CalculationMethod=3) then

			  (
if(cbc:MultiplierFactorNumeric and cbc:BaseAmount)  then 
			  (
if (( ../local-name()='Invoice') or ( ../local-name()='CreditNote'))  then 
			   (if(//cac-pl:AdditionalInvoiceGrossData/cac-pl:AllowanceCharge
				 [cbc-pl:AllowanceChargeReasonCode = $rAllowanceChargeReasonCode]
			 	 [cbc-pl:AllowanceChargeReason = $rAllowanceChargeReason][cac-pl:TaxCategory/cbc-pl:ID = $rID]
			 	 [cac-pl:TaxCategory/cbc-pl:Percent = $rPercent]
					/cac-pl:TaxTotal/cbc-pl:TaxInclusiveAmount) then 
			 		 (
						  if(round(//cac-pl:AdditionalInvoiceGrossData/cac-pl:AllowanceCharge
			  	 [cbc-pl:AllowanceChargeReasonCode = $rAllowanceChargeReasonCode]
			 	 [cbc-pl:AllowanceChargeReason = $rAllowanceChargeReason][cac-pl:TaxCategory/cbc-pl:ID = $rID]
			 	 [cac-pl:TaxCategory/cbc-pl:Percent = $rPercent]
					/cac-pl:TaxTotal/cbc-pl:TaxInclusiveAmount,2) = 
						round((xs:decimal(cbc:BaseAmount) * xs:decimal(cbc:MultiplierFactorNumeric) div 100),2)) 
						then true() else false()
					)
					 else 
			  		true()
					)
					 else 
				  (
if ((../ ../local-name()='Invoice') or (../../local-name()='CreditNote'))  then 
			   (if(//cac-pl:AdditionalInvoiceGrossData/cac-pl:InvoiceLine/cac-pl:AllowanceCharge
				 [cbc-pl:AllowanceChargeReasonCode = $rAllowanceChargeReasonCode]
			 	 [cbc-pl:AllowanceChargeReason = $rAllowanceChargeReason]
			 	 
					/cac-pl:TaxTotal/cbc-pl:TaxInclusiveAmount) then 
			 		 (
						  if(round(//cac-pl:AdditionalInvoiceGrossData/cac-pl:InvoiceLine/cac-pl:AllowanceCharge
			  	 [cbc-pl:AllowanceChargeReasonCode = $rAllowanceChargeReasonCode]
			 	 [cbc-pl:AllowanceChargeReason = $rAllowanceChargeReason]
			 	 
					/cac-pl:TaxTotal/cbc-pl:TaxInclusiveAmount,2) = 
						round((xs:decimal(cbc:BaseAmount) * xs:decimal(cbc:MultiplierFactorNumeric) div 100),2)) 
						then true() else false()
					)
					 else 
			  		true()
					)
					 else 
			  		true()
					)
					)
					 else 
			  		true()    
				)
		 else 
				(not(cbc:MultiplierFactorNumeric and cbc:BaseAmount) or u:slack(if (cbc:Amount) then cbc:Amount else 0, (xs:decimal(cbc:BaseAmount) * xs:decimal(cbc:MultiplierFactorNumeric)) div 100, 0.02))
            " flag="fatal">[PEPPOL-EN16931-R040] (1,2)Allowance/charge amount must equal base amount * percentage/100 if base amount and percentage exists 
								(3)Allowance/charge TaxInclusiveAmount must equal base amount * percentage/100 if base amount and percentage exists</assert>
    </rule>



    <!-- Payment -->
    <rule context="cac:PaymentMeans[some $code in tokenize('49 59', '\s') satisfies normalize-space(cbc:PaymentMeansCode) = $code]">
      <assert id="PLPEF-EN16931-R061"
              test="cac:PaymentMandate/cbc:ID"
              flag="fatal">Mandate reference MUST be provided for direct debit.</assert>
    </rule>

    <!-- Currency -->
    <rule context="cbc:Amount | cbc:BaseAmount | cbc:PriceAmount | cac:TaxTotal[cac:TaxSubtotal]/cbc:TaxAmount | cbc:TaxableAmount | cbc:LineExtensionAmount | cbc:TaxExclusiveAmount | cbc:TaxInclusiveAmount | cbc:AllowanceTotalAmount | cbc:ChargeTotalAmount | cbc:PrepaidAmount | cbc:PayableRoundingAmount | cbc:PayableAmount">
      <assert id="PLPEF-EN16931-R051"
              test="@currencyID = $documentCurrencyCode"
              flag="fatal">All currencyID attributes must have the same value as the invoice currency code (BT-5),  except for the invoice total VAT amount in accounting currency (BT-111) </assert>
    </rule>
    <rule context="//cac:TaxTotal[not(cac:TaxSubtotal)]/cbc:TaxAmount[@currencyID != //cbc:DocumentCurrencyCode]">
      <assert id="PLPEF-EN16931-R052"
              test="/*/cbc:TaxCurrencyCode and @currencyID = /*/cbc:TaxCurrencyCode"
              flag="fatal">The currencyID for invoice total VAT in accounting currency, must be the same as VAT accounting currency code (BT-6)</assert>
    </rule>

    <!-- Line level - invoice period -->
    <rule context="ubl-invoice:Invoice[cac:InvoicePeriod/cbc:StartDate]/cac:InvoiceLine/cac:InvoicePeriod/cbc:StartDate | ubl-creditnote:CreditNote[cac:InvoicePeriod/cbc:StartDate]/cac:CreditNoteLine/cac:InvoicePeriod/cbc:StartDate">
      <assert id="PLPEF-EN16931-R110"
              test="xs:date(text()) &gt;= xs:date(../../../cac:InvoicePeriod/cbc:StartDate)"
              flag="fatal">Start date of line period MUST be within invoice period.</assert>
    </rule>
    <rule context="ubl-invoice:Invoice[cac:InvoicePeriod/cbc:EndDate]/cac:InvoiceLine/cac:InvoicePeriod/cbc:EndDate | ubl-creditnote:CreditNote[cac:InvoicePeriod/cbc:EndDate]/cac:CreditNoteLine/cac:InvoicePeriod/cbc:EndDate">
      <assert id="PLPEF-EN16931-R111"
              test="xs:date(text()) &lt;= xs:date(../../../cac:InvoicePeriod/cbc:EndDate)"
              flag="fatal">End date of line period MUST be within invoice period.</assert>
    </rule>

    <!-- Line level - line extension amount -->
    <rule context="cac:InvoiceLine | cac:CreditNoteLine">
      <let name="lineExtensionAmount" value="if (cbc:LineExtensionAmount) then xs:decimal(cbc:LineExtensionAmount) else 0"/>
      <let name="quantity" value="if (../cac:InvoiceLine) then (if (cbc:InvoicedQuantity) then xs:decimal(cbc:InvoicedQuantity) else 1) else (if (cbc:CreditedQuantity) then xs:decimal(cbc:CreditedQuantity) else 1)"/>
      <let name="priceAmount" value="if (cac:Price/cbc:PriceAmount) then xs:decimal(cac:Price/cbc:PriceAmount) else 0"/>
      <let name="baseQuantity" value="if (cac:Price/cbc:BaseQuantity and xs:decimal(cac:Price/cbc:BaseQuantity) != 0) then xs:decimal(cac:Price/cbc:BaseQuantity) else 1"/>
      <let name="allowancesTotal" value="if (cac:AllowanceCharge[normalize-space(cbc:ChargeIndicator) = 'false']) then xs:decimal(sum(cac:AllowanceCharge[normalize-space(cbc:ChargeIndicator) = 'false']/cbc:Amount)) else 0"/>
      <let name="chargesTotal" value="if (cac:AllowanceCharge[normalize-space(cbc:ChargeIndicator) = 'true']) then xs:decimal(sum(cac:AllowanceCharge[normalize-space(cbc:ChargeIndicator) = 'true']/cbc:Amount)) else 0"/>
      <let name="valueID" value="cbc:ID"/>    
      <let name="nettAmount" value="round(xs:decimal(//ext:UBLExtensions/ext:UBLExtension/ext:ExtensionContent/cac-pl:AdditionalInvoiceGrossData/cac-pl:InvoiceLine[cbc-pl:ID=$valueID]/cbc-pl:LineExtensionGrossAmount - //ext:UBLExtensions/ext:UBLExtension/ext:ExtensionContent/cac-pl:AdditionalInvoiceGrossData/cac-pl:InvoiceLine[cbc-pl:ID=$valueID]/cac-pl:TaxTotal/cbc-pl:TaxAmount),2)"/>
                  
      <assert id="PEPPOL-EN16931-R120a" test="
      		  if(//ext:UBLExtensions/ext:UBLExtension/ext:ExtensionContent/cac-pl:AdditionalInvoiceGrossData/cbc-pl:CalculationMethod=3) then
				 (u:slack($lineExtensionAmount, $nettAmount, 0.02))
			  else
				 ( u:slack($lineExtensionAmount, ($quantity * ($priceAmount div $baseQuantity)) + $chargesTotal - $allowancesTotal, 0.02))
			 " flag="fatal">[PEPPOL-EN16931-R120a] (1,2) Invoice line net amount MUST equal (Invoiced quantity * (Item net price/item price base quantity) + Sum of invoice line charge amount - sum of invoice line allowance amount 
			 (3) Invoice line net amount MUST equal (Invoice Gross amount ) - Tax Gross amount
			 </assert>
      <assert id="PLPEF-EN16931-R121"
              test="not(cac:Price/cbc:BaseQuantity) or xs:decimal(cac:Price/cbc:BaseQuantity) &gt; 0"
              flag="fatal">Base quantity MUST be a positive number above zero.</assert>
    </rule>

    <!-- Allowance (price level) -->
    <rule context="cac:Price/cac:AllowanceCharge">
      <assert id="PLPEF-EN16931-R044"
              test="normalize-space(cbc:ChargeIndicator) = 'false'"
              flag="fatal">Charge on price level is NOT allowed.</assert>
      <assert id="PLPEF-EN16931-R046"
              test="not(cbc:BaseAmount) or xs:decimal(../cbc:PriceAmount) = xs:decimal(cbc:BaseAmount) - xs:decimal(cbc:Amount)"
              flag="fatal">Item net price MUST equal (Gross price - Allowance amount) when gross price is provided.</assert>
    </rule>

    <!-- Price -->
    <rule context="cac:Price/cbc:BaseQuantity[@unitCode]">
      <let name="hasQuantity" value="../../cbc:InvoicedQuantity or ../../cbc:CreditedQuantity"/>
<!--     <let name="quantity" value="if (/ubl-invoice:Invoice) then ../../cbc:InvoicedQuantity else ../../cbc:CreditedQuantity"/>		-->

      <assert id="PEPPOL-EN16931-R130" test="not($hasQuantity) or @unitCode = ../../cbc:InvoicedQuantity/@unitCode or @unitCode = ../../cbc:CreditedQuantity/@unitCode" flag="fatal">Unit code of price base quantity MUST be same as invoiced quantity.</assert>
  
    </rule>

  </pattern>

  <!-- National rules -->
  <pattern>

    <!-- NORWAY -->

    <rule context="cac:AccountingSupplierParty/cac:Party[$supplierCountry = 'NO']">
      <assert id="NO-R-001"
              test="cac:PartyLegalEntity"
              flag="fatal">Norwegian suppliers MUST provide legal entity.</assert>
      <assert id="NO-R-002"
              test="normalize-space(cac:PartyTaxScheme[normalize-space(cac:TaxScheme/cbc:ID) = 'TAX']/cbc:CompanyID) = 'Foretaksregisteret'"
              flag="warning">Most invoice issuers are required to append "Foretaksregisteret" to their invoice. "Dersom selger er aksjeselskap, allmennaksjeselskap eller filial av utenlandsk selskap skal også ordet «Foretaksregisteret» fremgå av salgsdokumentet, jf. foretaksregisterloven § 10-2."</assert>
    </rule>

    <!-- DENMARK -->

    <rule context="ubl-creditnote:CreditNote | ubl-invoice:Invoice">
      <assert id="DK-R-001"
              test="not(cac:AccountingSupplierParty/cac:Party/cac:PostalAddress/cac:Country/cbc:IdentificationCode = 'DK'
                        and (normalize-space(cbc:AccountingCost/text()) = ''))"
              flag="warning">For Danish suppliers when the Accounting code is known, it should be referred on the Invoice.</assert>
      <assert id="DK-R-002"
              test="not(cac:AccountingSupplierParty/cac:Party/cac:PostalAddress/cac:Country/cbc:IdentificationCode = 'DK'
                        and (normalize-space(./cac:AccountingSupplierParty/cac:Party/cac:PartyLegalEntity/cbc:CompanyID/text()) = ''))"
              flag="fatal">Danish suppliers MUST provide legal entity (CVR-number).</assert>
    </rule>

    <!-- Line level -->
    <rule context="ubl-creditnote:CreditNote/cac:CreditNoteLine | ubl-invoice:Invoice/cac:InvoiceLine">
      <assert id="DK-R-003"
              test="not(../cac:AccountingSupplierParty/cac:Party/cac:PostalAddress/cac:Country/cbc:IdentificationCode = 'DK'
                        and cac:Item/cac:CommodityClassification/cbc:ItemClassificationCode/@listID='MP'
                        and not(cac:Item/cac:CommodityClassification/cbc:ItemClassificationCode/@listVersionID = '19.05.01'
                               or cac:Item/cac:CommodityClassification/cbc:ItemClassificationCode/@listVersionID = '19.0501'
                            )
                        )"
              flag="fatal">If ItemClassification is provided from Danish suppliers, UNSPSC version 19.0501 should be used.</assert>
    </rule>

    <!-- Mix level -->
    <rule context="cac:AllowanceCharge">
      <assert id="DK-R-004"
              test="not(//cac:AccountingSupplierParty/cac:Party/cac:PostalAddress/cac:Country/cbc:IdentificationCode = 'DK'
                  and cbc:AllowanceChargeReasonCode = 'ZZZ'
                  and not(string-length(normalize-space(cbc:AllowanceChargeReason/text())) = 4
                          and number(cbc:AllowanceChargeReason) &gt;= 0
                          and number(cbc:AllowanceChargeReason) &lt;= 9999)
                 )"
              flag="fatal">When specifying non-VAT Taxes, Danish suppliers MUST use the AllowanceChargeReasonCode="ZZZ" and the 4-digit Tax category MUST be specified in 'AllowanceChargeReason'.</assert>
    </rule>

  </pattern>

  <!-- Restricted code lists and formatting -->
  <pattern>
    <let name="ISO3166" value="tokenize('AD AE AF AG AI AL AM AO AQ AR AS AT AU AW AX AZ BA BB BD BE BF BG BH BI BJ BL BM BN BO BQ BR BS BT BV BW BY BZ CA CC CD CF CG CH CI CK CL CM CN CO CR CU CV CW CX CY CZ DE DJ DK DM DO DZ EC EE EG EH ER ES ET FI FJ FK FM FO FR GA GB GD GE GF GG GH GI GL GM GN GP GQ GR GS GT GU GW GY HK HM HN HR HT HU ID IE IL IM IN IO IQ IR IS IT JE JM JO JP KE KG KH KI KM KN KP KR KW KY KZ LA LB LC LI LK LR LS LT LU LV LY MA MC MD ME MF MG MH MK ML MM MN MO MP MQ MR MS MT MU MV MW MX MY MZ NA NC NE NF NG NI NL NO NP NR NU NZ OM PA PE PF PG PH PK PL PM PN PR PS PT PW PY QA RE RO RS RU RW SA SB SC SD SE SG SH SI SJ SK SL SM SN SO SR SS ST SV SX SY SZ TC TD TF TG TH TJ TK TL TM TN TO TR TT TV TW TZ UA UG UM US UY UZ VA VC VE VG VI VN VU WF WS YE YT ZA ZM ZW 1A XI', '\s')"/>
    <let name="ISO4217" value="tokenize('AED AFN ALL AMD ANG AOA ARS AUD AWG AZN BAM BBD BDT BGN BHD BIF BMD BND BOB BOV BRL BSD BTN BWP BYN BZD CAD CDF CHE CHF CHW CLF CLP CNY COP COU CRC CUC CUP CVE CZK DJF DKK DOP DZD EGP ERN ETB EUR FJD FKP GBP GEL GHS GIP GMD GNF GTQ GYD HKD HNL HRK HTG HUF IDR ILS INR IQD IRR ISK JMD JOD JPY KES KGS KHR KMF KPW KRW KWD KYD KZT LAK LBP LKR LRD LSL LYD MAD MDL MGA MKD MMK MNT MOP MRO MUR MVR MWK MXN MXV MYR MZN NAD NGN NIO NOK NPR NZD OMR PAB PEN PGK PHP PKR PLN PYG QAR RON RSD RUB RWF SAR SBD SCR SDG SEK SGD SHP SLL SOS SRD SSP STN SVC SYP SZL THB TJS TMT TND TOP TRY TTD TWD TZS UAH UGX USD USN UYI UYU UZS VEF VND VUV WST XAF XAG XAU XBA XBB XBC XBD XCD XDR XOF XPD XPF XPT XSU XTS XUA XXX YER ZAR ZMW ZWL', '\s')"/>
    <let name="MIMECODE" value="tokenize('application/pdf image/png image/jpeg text/csv application/vnd.openxmlformats-officedocument.spreadsheetml.sheet application/vnd.oasis.opendocument.spreadsheet', '\s')"/>
    <let name="UNCL2005" value="tokenize('3 35 432', '\s')"/>
    <let name="UNCL5189" value="tokenize('41 42 60 62 63 64 65 66 67 68 70 71 88 95 100 102 103 104 105', '\s')"/>
    <let name="UNCL7161" value="tokenize('AA AAA AAC AAD AAE AAF AAH AAI AAS AAT AAV AAY AAZ ABA ABB ABC ABD ABF ABK ABL ABN ABR ABS ABT ABU ACF ACG ACH ACI ACJ ACK ACL ACM ACS ADC ADE ADJ ADK ADL ADM ADN ADO ADP ADQ ADR ADT ADW ADY ADZ AEA AEB AEC AED AEF AEH AEI AEJ AEK AEL AEM AEN AEO AEP AES AET AEU AEV AEW AEX AEY AEZ AJ AU CA CAB CAD CAE CAF CAI CAJ CAK CAL CAM CAN CAO CAP CAQ CAR CAS CAT CAU CAV CAW CAX CAY CAZ CD CG CS CT DAB DAC DAD DAF DAG DAH DAI DAJ DAK DAL DAM DAN DAO DAP DAQ DL EG EP ER FAA FAB FAC FC FH FI GAA HAA HD HH IAA IAB ID IF IR IS KO L1 LA LAA LAB LF MAE MI ML NAA OA PA PAA PC PL RAB RAC RAD RAF RE RF RH RV SA SAA SAD SAE SAI SG SH SM SU TAB TAC TT TV V1 V2 WH XAA YY ZZZ', '\s')"/>
    <let name="UNCL5305" value="tokenize('AE E S Z G O K L M', '\s')"/>

    <rule context="cbc:EmbeddedDocumentBinaryObject[@mimeCode]">
      <assert id="PLPEF-EN16931-CL001"
              test="some $code in $MIMECODE satisfies @mimeCode = $code"
              flag="fatal">Invalid mime code.</assert>
    </rule>

    <rule context="cac:AllowanceCharge[cbc:ChargeIndicator='false']/cbc:AllowanceChargeReasonCode">
      <assert id="PLPEF-EN16931-CL002"
              test="some $code in $UNCL5189 satisfies normalize-space(text()) = $code"
              flag="fatal">Reason code MUST be according to subset of UNCL 5189 D.16B.</assert>
    </rule>

    <rule context="cac:AllowanceCharge[cbc:ChargeIndicator='true']/cbc:AllowanceChargeReasonCode">
      <assert id="PLPEF-EN16931-CL003"
              test="some $code in $UNCL7161 satisfies normalize-space(text()) = $code"
              flag="fatal">Reason code MUST be according to UNCL 7161 D.16B.</assert>
    </rule>

    <rule context="cac:ClassifiedTaxCategory/cbc:ID | cac:TaxCategory/cbc:ID">
      <assert id="PLPEF-EN16931-CL004"
              test="some $code in $UNCL5305 satisfies normalize-space(text()) = $code"
              flag="fatal">Tax category code MUST be according to defined subset of UNCL 5305 D.16B.</assert>
    </rule>

    <rule context="cac:Country/cbc:IdentificationCode | cac:OriginCountry/cbc:IdentificationCode">
      <assert id="PLPEF-EN16931-CL005"
              test="some $code in $ISO3166 satisfies text() = $code"
              flag="fatal">Country code MUST be according to ISO 3166 Alpha-2.</assert>
    </rule>

    <rule context="cac:InvoicePeriod/cbc:DescriptionCode">
      <assert id="PLPEF-EN16931-CL006"
              test="some $code in $UNCL2005 satisfies normalize-space(text()) = $code"
              flag="fatal">Invoice period description code must be according to UNCL 2005 D.16B.</assert>
    </rule>

    <rule context="cbc:Amount | cbc:BaseAmount | cbc:PriceAmount | cbc:TaxAmount | cbc:TaxableAmount | cbc:LineExtensionAmount | cbc:TaxExclusiveAmount | cbc:TaxInclusiveAmount | cbc:AllowanceTotalAmount | cbc:ChargeTotalAmount | cbc:PrepaidAmount | cbc:PayableRoundingAmount | cbc:PayableAmount">
      <assert id="PLPEF-EN16931-CL007"
              test="some $code in $ISO4217 satisfies @currencyID = $code"
              flag="fatal">Currency code must be according to ISO 4217:2005</assert>
    </rule>

    <rule context="cbc:IssueDate | cbc:DueDate | cbc:TaxPointDate | cbc:StartDate | cbc:EndDate | cbc:ActualDeliveryDate">
      <assert id="PLPEF-EN16931-F001"
              test="string-length(text()) = 10 and (string(.) castable as xs:date)"
              flag="fatal">A date MUST be formatted YYYY-MM-DD.</assert>
    </rule>
  </pattern>

</schema>
